// **********************************************************************
//	Stereo.cpp  
//		Testes para fgeraop de imagens stereoscpicas
//
// **********************************************************************

#include <windows.h>
#include <stdio.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <glut.h>


#define ESQUERDO 1
#define DIREITO 2


// ************************************************
//   void ChangeResolution(int vmode)
//
// ************************************************
#define VM_640x480		1
#define VM_800x600		2
#define VM_1024x768		3
#define VM_1400x1050	4


void ChangeResolution(int vmode)
{
	DEVMODE dm;
	int ret;


	dm.dmSize = sizeof(DEVMODE);

	
	EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dm);


	switch (vmode)
	{ 
		case VM_640x480:
			dm.dmPelsWidth = 640;
			dm.dmPelsHeight = 480;
			break;
		case VM_800x600:
			dm.dmPelsWidth = 800;
			dm.dmPelsHeight = 600;
			break;
		case VM_1024x768:
			dm.dmPelsWidth = 1024;
			dm.dmPelsHeight = 768;
			break;
		case VM_1400x1050:
			dm.dmPelsWidth = 1400;
			dm.dmPelsHeight = 1050;
			break;
		default:
			printf("Invalid video mode\n");
			return;
			break;
	}
  
	ret = ChangeDisplaySettings(&dm, CDS_FULLSCREEN | CDS_TEST);
	switch (ret)
	{
		case DISP_CHANGE_SUCCESSFUL: 
			printf("The settings change was successful\n");
			ret = ChangeDisplaySettings(&dm, CDS_FULLSCREEN);//CDS_UPDATEREGISTRY);
			switch(ret)
			{
				case DISP_CHANGE_SUCCESSFUL: 
					printf("Resolution successfully changed!\n");
					break;
				case DISP_CHANGE_RESTART:
					printf("A reboot is necessary before the changes will take effect");
					break;
				default: printf ("Unable to change resolution!");
			}
			break;
		case DISP_CHANGE_RESTART: 
			printf("The computer must be restarted in order for the graphics mode to work.\n");
			break;
		case DISP_CHANGE_BADFLAGS:
			printf("An invalid set of flags was passed in.\n");
			break;
		case DISP_CHANGE_BADPARAM: 
			printf("An invalid parameter was passed in.\n");
			break;
		case DISP_CHANGE_FAILED:
			printf("The display driver failed the specified graphics mode.\n");
			break;
		case DISP_CHANGE_BADMODE:
			printf("The graphics mode is not supported.\n"); 
			break;
		case DISP_CHANGE_NOTUPDATED: 
			printf("Unable to write settings to the registry.\n");
			break;

	}
}


GLfloat ratio, DistOlhos = 0.55f;

// **********************************************************************
//  void DefineLuz(void)
//
//
// **********************************************************************
void DefineLuz(void)
{
	// Define cores para um objeto dourado
	GLfloat LuzAmbiente[]   = {0.24725f, 0.1995f, 0.07f } ;
	GLfloat LuzDifusa[]   = {0.75164f, 0.60648f, 0.22648f, 1.0f };
	GLfloat LuzEspecular[] = {0.626281f, 0.555802f, 0.366065f, 1.0f };
	GLfloat PosicaoLuz0[]  = {3.0f, 3.0f, 0.0f, 1.0f };
	GLfloat PosicaoLuz1[]  = {-3.0f, -3.0f, 0.0f, 1.0f };
	GLfloat Especularidade[] = {1.0f, 1.0f, 1.0f, 1.0f };

// ****************  Fonte de Luz 0

	glEnable ( GL_COLOR_MATERIAL );

	// Habilita o uso de iluminao
	glEnable(GL_LIGHTING);

	// Ativa o uso da luz ambiente
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);
	// Define os parametros da Luz nmero Zero
	glLightfv(GL_LIGHT0, GL_AMBIENT, LuzAmbiente);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, LuzDifusa  );
	glLightfv(GL_LIGHT0, GL_SPECULAR, LuzEspecular  );
	glLightfv(GL_LIGHT0, GL_POSITION, PosicaoLuz0 );
	glEnable(GL_LIGHT0);

	// Ativa o "Color Tracking"
	glEnable(GL_COLOR_MATERIAL);

	// Define a reflectancia do material
	glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

	// Define a concentrao do brilho.
	// Quanto maior o valor do Segundo parametro, mais
	// concentrado ser o brilho. (Valores vlidos: de 0 a 128)
	glMateriali(GL_FRONT,GL_SHININESS,51.2);

// ****************  Fonte de Luz 1

	// Ativa o uso da luz ambiente
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);
	// Define os parametros da Luz nmero Zero
	glLightfv(GL_LIGHT1, GL_AMBIENT, LuzAmbiente);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LuzDifusa  );
	glLightfv(GL_LIGHT1, GL_SPECULAR, LuzEspecular  );
	glLightfv(GL_LIGHT1, GL_POSITION, PosicaoLuz1 );
	glEnable(GL_LIGHT1);

	// Ativa o "Color Tracking"
	glEnable(GL_COLOR_MATERIAL);

	// Define a reflectancia do material
	glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

	// Define a concentrao do brilho.
	// Quanto maior o valor do Segundo parametro, mais
	// concentrado ser o brilho. (Valores vlidos: de 0 a 128)
	glMateriali(GL_FRONT,GL_SHININESS,20);

}


// **********************************************************************
//  void init(void)
//		Inicializa os parmetros globais de OpenGL
//
// **********************************************************************
void Init(void)
{
	glClearColor(1.0f, 1.0f, 0.0f, 1.0f); // Fundo de tela preto

	glShadeModel(GL_SMOOTH);
	
	glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_DEPTH_TEST);
	glEnable ( GL_CULL_FACE );
	
}


// **********************************************************************
//  void DrawLine(int x1, int y1, int x2, int y2)
//
//
// **********************************************************************
void DrawLine(int x1, int y1, int x2, int y2)
{
	glVertex2f(x1,y1);
	glVertex2f(x2,y2);
}

// **********************************************************************
//  void DesenhaFundo()
//      Inicializa o stencil com 1 nas linhas pares 
//
// **********************************************************************
void DesenhaFundo(int Largura, int Altura )
{

	glMatrixMode(GL_PROJECTION); // 
	glLoadIdentity ();
	gluOrtho2D(0, Largura, 0, Altura);

	glMatrixMode(GL_MODELVIEW);

	glBegin(GL_LINES);
		for (int y= 0; y < Altura; y += 2)
		{
			DrawLine(0,y, Largura, y);
		}
	glEnd();
	// volta  matriz de objetos
	//glMatrixMode(GL_MODELVIEW);

}

// **********************************************************************
//  void DrawStencil()
//     
//
// **********************************************************************
void DrawStencil(int Largura, int Altura )
{
    // Habilita o uso do Stencil 
	glEnable(GL_STENCIL_TEST);

	// Define que "0" ser usado para limpar o Stencil 
	glClearStencil(0);
	// limpa o Stencil
	glClear(GL_STENCIL_BUFFER_BIT);

	glStencilFunc(GL_ALWAYS, 1, 1); 
	glStencilOp(GL_REPLACE,GL_REPLACE,GL_REPLACE);
	
	DesenhaFundo(Largura,Altura);

}

// **********************************************************************
//  void reshape( int Largura, int Altura )
//		trata o redimensionamento da janela OpenGL.
//		Atualiza o stencil buffer 
// **********************************************************************
void reshape( int Largura, int Altura )
{
    
	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(Altura == 0)
		Altura = 1;

	ratio = 1.0f * Largura / Altura;
	// 
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, Largura, Altura);

	DrawStencil(Largura, Altura);
}
// **********************************************************************
//  void DesenhaCubo()
//
//
// **********************************************************************
void DesenhaCubo()
{
	glBegin ( GL_QUADS );
		// Front Face
		glNormal3f(0,0,1);
		glVertex3f(-1.0f, -1.0f,  1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);
		// Back Face
		glNormal3f(0,0,-1);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		// Top Face
		glNormal3f(0,1,0);
		glVertex3f(-1.0f,  1.0f, -1.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		// Bottom Face
		glNormal3f(0,-1,0);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);
		// Right face
		glNormal3f(1,0,0);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		// Left Face
		glNormal3f(-1,0,0);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);
	glEnd();
}

// **********************************************************************
//  void DesabilitaDesenhoNoStencil()
//
//
// **********************************************************************
void DesabilitaDesenhoNoStencil()
{
	glStencilFunc(GL_NOTEQUAL, 1, 1);   
	glStencilOp(GL_KEEP,GL_KEEP, GL_KEEP);
}

// **********************************************************************
//  void HabilitaDesenhoNoStencil()
//
//
// **********************************************************************
void HabilitaDesenhoNoStencil()
{

	glStencilFunc(GL_EQUAL, 1, 1);   
	glStencilOp(GL_KEEP,GL_KEEP, GL_KEEP);
}
// **********************************************************************
//  void PosicUser()
//		esta funo define a posicde cada um dos olhos do observador 
//
// **********************************************************************
void PosicUser(int Olho)
{
	// Set the clipping volume
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(80,ratio,0.01,200);

	switch (Olho)
	{
	case ESQUERDO:
				gluLookAt(-DistOlhos/2, 0, 5, 
						  0, 0, 0,
			              0.0f,1.0f,0.0f);
				break;
	case DIREITO:
				gluLookAt(DistOlhos/2, 0, 5, 
						  0, 0, 0,
			              0.0f,1.0f,0.0f);
				break;
	}
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

}

// **********************************************************************
//  void DesenhaCena()
//  
//
// **********************************************************************
void DesenhaCena()
{

	static GLfloat ang = 0;

	glMatrixMode(GL_MODELVIEW);

	//glLoadIdentity ();
	glPushMatrix();
//		glTranslatef ( 1.0f, 0.0f, 0.0f );
//		glRotatef(ang,0,1,0);
//		glRotatef(ang,1,0,0);

		glRotatef(ang,0,1,0);
		glTranslatef ( 2.5f, 0.0f, 0.0f );
		glColor3f(0.5f,0.3f,0.0f);
		DesenhaCubo();		
	glPopMatrix();


	glPushMatrix();
//		glTranslatef ( -1.0f, 1.0f, -5.0f );
		glRotatef(45,0,1,0);
		glColor3f(0.5f,0.3f,0.0f);
		DesenhaCubo();		
	glPopMatrix();


	ang = ang + 1;

}
// **********************************************************************
//  void display( void )
//
//
// **********************************************************************
void display( void )
{

	// Limpa a tela
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	DefineLuz();

	PosicUser(ESQUERDO);
		DesabilitaDesenhoNoStencil();
		DesenhaCena();

	PosicUser(DIREITO);
		HabilitaDesenhoNoStencil();
		DesenhaCena();

	glutSwapBuffers();

}

// **********************************************************************
//  void keyboard ( unsigned char key, int x, int y )
//
//
// **********************************************************************
void keyboard ( unsigned char key, int x, int y )  
{
	switch ( key ) 
	{
    case 27:        // Termina o programa qdo
      exit ( 0 );   // a tecla ESC for pressionada
      break;        
	
    default:        
      break;
  }
}

// **********************************************************************
//  void arrow_keys ( int a_keys, int x, int y )  
//
//
// **********************************************************************
void arrow_keys ( int a_keys, int x, int y )  
{
	switch ( a_keys ) 
	{
		case GLUT_KEY_UP:       
				DistOlhos += 0.005f;				
			break;
	    case GLUT_KEY_DOWN:     
				DistOlhos -= 0.005f;
			break;
		default:
			break;
	}
	if (DistOlhos < 0 )
		DistOlhos = 0;
	printf("Distancia entre os olhos: %6.3f\n", DistOlhos);
}

// **********************************************************************
//  void main ( int argc, char** argv )
//
//
// **********************************************************************
void main ( int argc, char** argv )   
{

	//ChangeResolution(VM_640x480);

	glutInit            ( &argc, argv ); 
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL); 
	glutInitWindowPosition (0,0);
	glutInitWindowSize  ( 640,480 ); 
	glutCreateWindow    ( "Tpicos em Computao Grfica - Teste com Imagens Stereoscpicas." ); 

	//glutFullScreen();	
	Init ();

	glutDisplayFunc ( display );  
	glutReshapeFunc ( reshape );
	glutKeyboardFunc ( keyboard );
	glutSpecialFunc ( arrow_keys );
	glutIdleFunc ( display );

	glutMainLoop ( );          
}




